function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
import { useContext, useState, useEffect, useCallback, useImperativeHandle } from 'react';
import { RouterContext, RouterViewContext } from '../context';
import { _checkActivate, _checkDeactivate } from '../router-view';
import { innumerable, isFunction, isPlainObject, isNumber, readRouteMeta, isEmptyRouteState } from '../util';
function isCommonPage(matched, commonPageName) {
  return Boolean(commonPageName && matched.some(r => readRouteMeta(r.config, commonPageName)));
}
function getRouteMatched(router, currentRoute, commonPageName) {
  if (!currentRoute || !router) return [];
  let {
    matched = []
  } = currentRoute;
  if (matched) {
    const _isCommonPage = isCommonPage(matched, commonPageName);
    if (_isCommonPage && currentRoute.query.redirect) {
      matched = router.getMatched(currentRoute.query.redirect);
    }
  }
  return matched;
}
function useRouter(defaultRouter) {
  // eslint-disable-next-line react-hooks/rules-of-hooks
  return defaultRouter || useContext(RouterContext);
}
function useRoute(defaultRouter, options = {}, anotherWatch = null) {
  const router = useRouter(defaultRouter);
  const [route, setRoute] = useState(router ? router.currentRoute || router.initialRoute : null);
  if (options.watch && router) {
    // eslint-disable-next-line react-hooks/rules-of-hooks
    const onRouteChange = useCallback(async (route, prevRoute, router) => {
      if (options.ignoreSamePath !== false && route && prevRoute && route.fullPath === prevRoute.fullPath) return;
      if (anotherWatch && (await anotherWatch(route, prevRoute, router)) === false) return;
      if (isFunction(options.watch) && (await options.watch(route, prevRoute, router)) === false) return;
      if (options.delay) setTimeout(() => setRoute(route), isNumber(options.delay) ? options.delay : 0);else setRoute(route);
    }, [options, anotherWatch]);
    // eslint-disable-next-line react-hooks/rules-of-hooks
    useRouteChanged(router, onRouteChange);
  }
  return route;
}
function useRouterView() {
  return useContext(RouterViewContext);
}
function useMatchedRouteIndex(matchedOffset = 0) {
  const view = useRouterView();
  const index = view ? view.state.depth : 0;
  return Math.max(0, index + matchedOffset);
}
function useMatchedRouteAndIndex(defaultRouter, options = {}) {
  const {
    matchedOffset
  } = options;
  const router = useRouter(defaultRouter);
  const routeIndex = useMatchedRouteIndex(matchedOffset);
  const matchRouteWatch = useCallback((route, prevRoute) => route.matched[routeIndex] !== prevRoute.matched[routeIndex], [routeIndex]);
  const route = useRoute(defaultRouter, options, matchRouteWatch);
  const matched = getRouteMatched(router, route, options.commonPageName);
  const matchedRoute = matched && matched[routeIndex] || null;
  return [matchedRoute, routeIndex];
}
function useMatchedRoute(defaultRouter, options) {
  const ret = useMatchedRouteAndIndex(defaultRouter, options);
  return ret[0];
}
function useRouteMeta(metaKey, defaultRouter, options) {
  const router = useRouter(defaultRouter);
  const route = useMatchedRoute(router, options);
  const meta = route && route.meta;
  const keyIsArray = Array.isArray(metaKey);
  if (keyIsArray && !metaKey.length) throw new Error('metaKey is Empty!');
  const [value, _setValue] = useState(() => keyIsArray ? metaKey.reduce((p, key) => {
    if (!meta) return p;
    p[key] = meta[key];
    return p;
  }, {}) : {
    [metaKey]: meta && meta[metaKey]
  });
  const setValue = useCallback((newValue, setAll = false) => {
    if (!meta) return;
    if (!keyIsArray) {
      newValue = {
        [metaKey]: newValue
      };
    } else if (!setAll) {
      newValue = Object.keys(newValue).reduce((p, key) => {
        if (metaKey.includes(key)) p[key] = newValue[key];
        return p;
      }, {});
    }
    const changed = route && router && router.updateRouteMeta(route, newValue, {
      ignoreConfigRoute: options?.ignoreConfigRoute
    });
    if (!changed) return;
    _setValue(_objectSpread(_objectSpread({}, value), newValue));
  }, [meta, keyIsArray, route, router, value, metaKey, options?.ignoreConfigRoute]);
  return keyIsArray ? [value, setValue] : [value[metaKey], setValue];
}
function useRouteState(defaultRouter, stateAction, options) {
  const [defaultState] = useState(stateAction || {});
  const router = useRouter(defaultRouter);
  const route = useMatchedRoute(router || defaultRouter, options);
  const routeUrl = route && route.url;
  const setRouteState = useCallback(newState => router && router.replaceState(newState, route || undefined),
  // eslint-disable-next-line react-hooks/exhaustive-deps
  [router, routeUrl]);
  let state = route && route.state;
  if (isEmptyRouteState(state) && defaultState) state = defaultState;
  return [state, setRouteState];
}
function useRouteChanged(router, onChange, deps = []) {
  const [plugin] = useState({});
  plugin.onRouteChange = onChange;
  return useEffect(() => {
    const unplugin = router.plugin(plugin);
    return () => unplugin && unplugin();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [router, plugin, ...deps]);
}
function useRouteMetaChanged(router, onChange, deps = []) {
  const [plugin] = useState({});
  plugin.onRouteMetaChange = useCallback((newVal, oldVal, route, router) => {
    if (deps.length && !deps.some(v => {
      if (isPlainObject(v)) return v === route.meta;
      return v in oldVal;
    })) return;
    return onChange(newVal, oldVal, route, router);
  }, [deps, onChange]);
  return useEffect(() => {
    const unplugin = router.plugin(plugin);
    return () => unplugin && unplugin();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [router, plugin, ...deps]);
}
function createRouteGuardsRef(ref) {
  if (ref && !ref.__routeGuardInfoHooks) innumerable(ref, '__routeGuardInfoHooks', true);
  return ref;
}
function useRouteGuardsRef(ref, guards, deps = []) {
  if (!isFunction(guards)) deps.concat(guards);
  return useImperativeHandle(ref, (...args) => {
    let ret;
    if (isFunction(guards)) ret = guards(...args);else ret = guards;
    return createRouteGuardsRef(ret);
  },
  // eslint-disable-next-line react-hooks/exhaustive-deps
  deps);
}
function useRouterViewEvent(name, onEvent, unshift) {
  const view = useRouterView();
  const [$refs] = useState({});
  $refs.onEvent = onEvent;
  $refs.unshift = unshift;
  useEffect(() => {
    if (!view) return;
    const fn = (...args) => $refs.onEvent(...args);
    view._events[name][$refs.unshift ? 'unshift' : 'push'](fn);
    return () => {
      const idx = view._events[name].indexOf(fn);
      if (~idx) view._events[name].splice(idx, 1);
    };
  }, [$refs, view, name]);
}
function useViewActivate(onEvent) {
  const router = useRouter();
  const current = useMatchedRoute();
  useRouterViewEvent('activate', event => {
    if (!onEvent || !_checkActivate(router, current, event)) return;
    onEvent(event);
  });
}
function useViewDeactivate(onEvent) {
  const router = useRouter();
  const current = useMatchedRoute();
  useRouterViewEvent('deactivate', event => {
    if (!onEvent || !_checkDeactivate(router, current, event)) return;
    onEvent(event);
  }, true);
}
export { isCommonPage, getRouteMatched, useRouter, useRouteChanged, useRouteMetaChanged, useRoute, useRouteMeta, useRouteState, useRouterView, useMatchedRouteIndex, useMatchedRoute, useMatchedRouteAndIndex, useRouteGuardsRef, useRouterViewEvent, useViewActivate, useViewDeactivate, createRouteGuardsRef };